#include <QApplication>
#include <QGraphicsView>
#include <QGraphicsSceneMouseEvent>
#include <QGraphicsProxyWidget>
#include <QPushButton>
#include <QVBoxLayout>
#include <QScriptEngine>
#include <QFile>
#include <QDebug>
#include <qmath.h>

#include "latenteditor.h"

//=================================================================================================
LatentEditor::LatentEditor( QWidget *parent ):
  QWidget( parent ),
  scene(0),
  normalPen( Qt::black, 2, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin ),
  selectedPen( Qt::blue, 2, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin ),
  dashPen( Qt::lightGray, 1, Qt::DashDotLine, Qt::RoundCap, Qt::RoundJoin )
{
  QScriptEngine engine;

  QGraphicsView *view = new QGraphicsView( this );
  view->setRenderHint( QPainter::Antialiasing, true );
  scene = new GraphicsScene( view );
  view->setScene( scene );

  scene->addEllipse( 255, 415, 90, 90, normalPen );
  scene->addEllipse( 210, 370, 180, 180, normalPen );
  scene->addRect( 50, 160, 500, 210, normalPen );
  scene->addLine( 205, 460, 395, 460, dashPen );
  scene->addLine( 300, 155, 300, 560, dashPen );
  scene->addLine( 175, 155, 175, 375, dashPen );
  scene->addLine( 425, 155, 425, 375, dashPen );

  QScriptValue obj = engine.newQObject( scene );
  engine.globalObject().setProperty( "Scene", obj );
  //engine.evaluate( "Scene.clear();" );

  QVBoxLayout *mainLayout = new QVBoxLayout( this );
  mainLayout->addWidget( view );
}
//=================================================================================================



//=================================================================================================
GraphicsScene::GraphicsScene( QWidget *parent ):
  QGraphicsScene( parent )
{
  setItemIndexMethod( QGraphicsScene::NoIndex );
  setSceneRect( 0.0, 0.0, 600.0, 600.0 );

  _180divPI = 180.0 / M_PI;
  PIdiv180 = M_PI / 180.0;
}
//=================================================================================================
void GraphicsScene::mousePressEvent( QGraphicsSceneMouseEvent *e )
{
  if (e->buttons() & Qt::RightButton)
  {
    Zacep *ellipse = new Zacep();
    addItem( ellipse );
    ellipse->setPos( 370, 435 );
  }

  QGraphicsScene::mousePressEvent(e);
}
//=================================================================================================
void GraphicsScene::mouseMoveEvent( QGraphicsSceneMouseEvent *e )
{
  if (e->buttons() & Qt::LeftButton)
  {
    QGraphicsItem *it = itemAt( e->scenePos() );

    if (it)
    {
      if (it->data( 777 ).toInt() == 777)
      {
        float mouse_radius = getDistanceBetweenPoints( CenterX, CenterY, e->scenePos().x(), e->scenePos().y() );

        float cos_stopAngle = float( e->scenePos().x() - CenterX ) / mouse_radius;

        if (cos_stopAngle > 1.0)
          cos_stopAngle = 1.0;
        if (cos_stopAngle < -1.0)
          cos_stopAngle = -1.0;

        float stopAngle = _180divPI * qAcos( cos_stopAngle );

        if (e->scenePos().y() > CenterY && e->scenePos().x() > CenterX)
          stopAngle = 360 - stopAngle;
        else if (e->scenePos().y() > CenterY)
          stopAngle = 360 - stopAngle;

        it->setTransformOriginPoint( it->mapFromScene( CenterX, CenterY ) );
        qDebug() << it->mapFromScene( CenterX, CenterY );
        it->setRotation( -stopAngle );
      }
    }
  }

  QGraphicsScene::mouseMoveEvent(e);
}
//=================================================================================================
float GraphicsScene::getVectorLength( float dx, float dy )
{
  return qSqrt( dx * dx + dy * dy );
}
//=================================================================================================
float GraphicsScene::getDistanceBetweenPoints( float x1, float y1, float x2, float y2 )
{
  return getVectorLength( x2 - x1, y2 - y1 );
}
//=================================================================================================



//=================================================================================================
Zacep::Zacep()
{
  setData( 777, 777 );
}
//=================================================================================================
QRectF Zacep::boundingRect() const
{
  return QRectF( 0, 0, 20, 50 );
}
//=================================================================================================
void Zacep::paint( QPainter *painter, const QStyleOptionGraphicsItem */*option*/, QWidget */*widget*/ )
{
  painter->drawLine( 5, 0, 16, 0 );
  painter->drawLine( 5, 7, 18, 7 );
  painter->drawLine( 5, 14, 18, 14 );
  painter->drawLine( 5, 21, 20, 21 );
  painter->drawLine( 5, 28, 20, 28 );
  painter->drawLine( 5, 35, 18, 35 );
  painter->drawLine( 5, 42, 18, 42 );
  painter->drawLine( 5, 49, 16, 49 );
}
//=================================================================================================
void Zacep::mousePressEvent( QGraphicsSceneMouseEvent *e )
{
}
//=================================================================================================
void Zacep::mouseMoveEvent( QGraphicsSceneMouseEvent *e )
{
}
//=================================================================================================
